var designAnalysisData;

var importanceClass = {
                        I : 0.8,
                        II : 1,
                        III : 1.2,
                        IV : 1.4
                        };

var groundType = {
                  A : "A",
                  B : "B",
                  C : "C", 
                  D : "D",
                  E : "E"
                    };

var groundTypeVals;

initGroundTypeVals();

var graphLineColors = {
                        design_spectrum : "#C71112",
                        elastic_spectrum : "#0000FF"
                        };

var groundTypeDesc = {
                        A : "Skala ali druga skali podobna geološka formacija, na kateri je "+
                            "največ 5 m slabšega površinskega materiala.",
                        B : "Zelo gost pesek, prod ali zelo toga glina, debeline vsaj nekaj deset "+
                            "metrov, pri katerih mehanske značilnosti z globino postopoma naraščajo.",
                        C : "Globoki sedimenti gostega ali srednje gostega peska, proda ali toge gline "+
                            "globine nekaj deset do več sto metrov.",
                        D : "Sedimenti rahlih do srednje gostih nevezljivih zemljin (z nekaj mehkimi "+
                            "vezljivimi plastmi plastmi ali brez njih) ali pretežno mehkih do trdih "+
                            "vezljivih zemljin.",
						E : "Profil tal, kjer površinska aluvialna plast debeline med okrog 5 in 20 metri "+
							"in vrednostmi vs, ki ustrezajo tipoma C ali D, leži na najbolj togem materialu "+
							"z vs > 800 m/s."
                        };
                        
var importanceClassDesc = {
                        I : "Stavbe manjše pomembnosti za varnost ljudi, npr.kmetijski objekti in podobno.",
                        II : "Običajne stavbe, ki ne sodijo v druge kategorije.",
                        III : "Stavbe, katerih potresna odpornost je pomembna glede na "+
                              "posledice porušitve, npr. šole, dvorane za srečanja, kulturne "+
                              "ustanove in podobno.",
                        IV : "Stavbe, katerih integriteta med potresi je življenskega pomena za "+
                             "civilno zaščito, npr. bolnišnice, gasilske postaje, elektrarne in podobno."
                        };

var spectrumGraphTitle = 'za q = {0}, tip tal = {1}, ag = {2}g, kategorija pomembnosti = {3}';

function drawSpectrum()
{
    var q = $("#qVal").val();
    var gr_type = $("#grTypeSelect").val();
    var importanceF = $("#impClassSelect").val();
    var agR = $("#accSelect").val();
    var title = String.format(spectrumGraphTitle, q, gr_type, agR, importanceF);
	$("#spectrumTitle").html(title);

    //object for curent ground type
    var ground_vals = groundTypeVals[gr_type];

    //prepare vibration periods
    var vibration_periods = prepareVibrationPeriods(ground_vals);

    var ag = importanceClass[importanceF] * agR;
    
	designAnalysisData = prepareDesignAnalysisData(vibration_periods, ground_vals, q, ag ); 
	
    var elasticAnalysisData = prepareElasticAnalysisData(vibration_periods, ag, ground_vals);
    	                                                        
    $("#tableData").html(outputTableData(designAnalysisData, elasticAnalysisData));
    	
	var spectrumChart = new JSChart('spectrumGraphics', 'line');

		spectrumChart.setDataArray(designAnalysisData, "designSpectrum");
		spectrumChart.setDataArray(elasticAnalysisData, "elasticSpectrum");
		spectrumChart.setAxisNameFontSize(10);
		spectrumChart.setAxisNameX('T(s)');
		spectrumChart.setAxisNameY('Sa-T- -g-');
		spectrumChart.setAxisNameColor('#A03');
		//spectrumChart.setAxisValuesNumberX(6);
		//spectrumChart.setAxisValuesNumberY(5);
		spectrumChart.setAxisValuesColor('#38a4d9');
		spectrumChart.setAxisColor('#38a4d9');
		spectrumChart.setLineColor(graphLineColors.design_spectrum, "designSpectrum");
		spectrumChart.setLineColor(graphLineColors.elastic_spectrum, "elasticSpectrum");
		spectrumChart.setTitle("");
		spectrumChart.setTitleColor('#A03');	
		spectrumChart.setGraphExtend(true);
		spectrumChart.setTitleFontSize(14);
		spectrumChart.setGridColor('#38a4d9');
		//spectrumChart.setSize(600, 320);
		spectrumChart.setSize(640, 450);
		spectrumChart.setAxisPaddingLeft(80);
		spectrumChart.setAxisPaddingRight(10);
		spectrumChart.setAxisPaddingTop(100);
		spectrumChart.setAxisPaddingBottom(45);
		spectrumChart.setTextPaddingLeft(30);
		spectrumChart.setTextPaddingBottom(10);

	spectrumChart.draw();
	showDiagramDiv();
}

function prepareDesignAnalysisData(vibrationPeriods, groundVals, q, ag)
{
    // T = 0
    var val0 = ag*(2*groundVals.S/3);
    
	// Tb < T < Tc
    var val1 = ag*groundVals.S*(2.5/q);
    
    var designAnalysisData = [
                                [0, val0],
                                [vibrationPeriods[0], val1],
                                [vibrationPeriods[1], val1]
                              ];
    
    //calculate points from Tc to Td
    var eq3points = vibrationPeriods[2];
    var val;
    var point;
    var beta = 0.2;
    for(var i=0; i < eq3points.length;i++)
    {
        point = eq3points[i];
       
        val = ag * groundVals.S * (2.5/q) * (groundVals.T_c/point);
		
			if (val < ag * beta) 
	 		{
		 		val = ag * beta;
	 		}
        
        designAnalysisData.push([point, val]);
    }
    
    // Td < T < 4
    var eq4points = vibrationPeriods[3];
    for(i=0;i<eq4points.length;i++)
    {
        point = eq4points[i];
      
        val = ag * groundVals.S * (2.5/q) * ((groundVals.T_c * groundVals.T_d)/(Math.pow(point, 2)));

			if (val < ag * beta) 
	 		{
		 		val = ag * beta;
	 		}
			
        designAnalysisData.push([point, val]);
    }
                              
    return designAnalysisData;
}

function prepareElasticAnalysisData(vibrationPeriods, ag, groundVals)
{
    var val0 = ag * groundVals.S;
    var val1 = ag * 2.5 * groundVals.S;
    
    var elasticAnalysisData = [
                                    [0, val0],
                                    [groundVals.T_b, val1],
                                    [groundVals.T_c, val1]
                              ];
    
    var point;
    var val;
	    
    // Tb < T < Tc
    var eq3points = vibrationPeriods[2];                  
    for(var i=0;i<eq3points.length;i++)
    {
        point = eq3points[i];
        val = ag * groundVals.S * 2.5 * (groundVals.T_c/point);

	
        elasticAnalysisData.push([point, val]);
    }
    
    // Td < T < 4
    var eq4points = vibrationPeriods[3];
    for(i=0;i<eq4points.length;i++)
    {
        point = eq4points[i];
        val = ag * groundVals.S * 2.5 * ((groundVals.T_c * groundVals.T_d)/Math.pow(point, 2));
  
        elasticAnalysisData.push([point, val]);
    }
    
	return elasticAnalysisData;
}

//creates the array of vibration periods (points) in which calculate the value of spectrums
//indices:
//0, 1 are only one point for Tb and Tc
//2 are points from Tc to Td
//3 are points from Td to 4s
function prepareVibrationPeriods(ground_vals)
{
    var vibration_periods = [ground_vals.T_b, ground_vals.T_c];
    var point = ground_vals.T_c;
    
    var tc_td_step = 0.1;
    var td_end_step = 0.1;
    
    //creates vibration period from T_c to T_d
    var eq3_points = [];
    while(point + tc_td_step < ground_vals.T_d)
    {
        point += tc_td_step;
        eq3_points.push(point);
    }
    eq3_points.push(ground_vals.T_d);
    vibration_periods.push(eq3_points);
    
    //creates vibration period from T_d do 4
    point = ground_vals.T_d;
    var eq4_points = [];
    while(point + td_end_step < 4)
    {
        point += td_end_step;
        eq4_points.push(point);
    }
    eq4_points.push(4);
    vibration_periods.push(eq4_points);
    
    return vibration_periods;
}

function outputTableData(designSpectrumData, elasticSpectrumData)
{
    var table = "<table>"+
                "<tr><th>T(s)</th>"+
                "<th style='color:"+graphLineColors.design_spectrum+"'>Sd(g)</th>"+
                "<th style='color:"+graphLineColors.elastic_spectrum+"'>Sae</th></tr>";
                
    //Index 0 contains the vibration period, index 1 contains the value
    for(var i=0;i<designSpectrumData.length;i++)
    {
        var design_values = designSpectrumData[i];
        var elastic_values = elasticSpectrumData[i];
        table += "<tr><td>"  + formatNumber(design_values[0], 2, true)+
				 "</td><td>" + formatNumber(design_values[1], 4, true)+
				 "</td><td>" + formatNumber(elastic_values[1], 4, true)+
				 "</td></tr>";
    }
                
    table += "</table>";
    
    return table;
}

//object for ground type
function GroundType(S, T_b, T_c, T_d)
{
    this.S = S;
    this.T_b = T_b;
    this.T_c = T_c;
    this.T_d = T_d;
}

function initGroundTypeVals()
{
    groundTypeVals = new Object();
    groundTypeVals[groundType.A] = new GroundType(1, 0.1, 0.4, 2);
    groundTypeVals[groundType.B] = new GroundType(1.2, 0.15, 0.5, 2);
    groundTypeVals[groundType.C] = new GroundType(1.15, 0.2, 0.6, 2);
    groundTypeVals[groundType.D] = new GroundType(1.35, 0.2, 0.8, 2);
    groundTypeVals[groundType.E] = new GroundType(1.7, 0.1, 0.4, 2);
}

//rounded numbers to n decimals
function formatNumber(number, n, pad)
{
    if(n == null)
    {
        n=4;
    }
    if(pad == null)
    {
        pad = false;
    }
    try
    {
        number = Number(number);
		number = Number(number.toFixed(n))
		if(pad)
		{
		    return padDecimalNumber(number, n);
		}
		else
		{
		    return number;
		}
    }
    catch(err)
    {
        return null;
    }
}

function padDecimalNumber(number, n)
{
    number = number.toString();
    var dotIndex = number.lastIndexOf(".");
    
    if(dotIndex == -1)
    {
        number += ".";
        dotIndex = number.length + 1;
        //return number;
    }
    
    var decimal_length = number.substring(dotIndex+1, number.length).length;
    while(n > decimal_length)
    {
        number += "0";
        decimal_length++;
    }
    return number;
}

//sprozeno ko se izbere nekaj iz selecta 'Tip tal' initiated when chosen something from select element 'Ground type'
//in div = "GroundDescription" prints properties of selected Ground type
function groundTypeSelected(form)
{
    var selected_value = form.Ground.value;
    var desc;
    if(selected_value == -1)
    {
        desc = "";
    }
    else
    {
        desc = groundTypeDesc[selected_value];
    }
    
    $("#GroundDescription").html(desc);
    updateSpectrumDiagram();
}

//sprozeno, ko se izbere nekaj iz selecta ImportanceClass
function categorySelected(form)
{
    var selected_value = form.ImportanceClass.value;    
    var desc;
    if(selected_value == -1)
    {
        desc = "";
    }
    else
    {
        desc = importanceClassDesc[selected_value];
    }
    
    $("#ImpClassDescription").html(desc);
    updateSpectrumDiagram();
}

//nastavi v selectu projektni pospesek tal, posredovan
//od mape
function selectAccelerationParameter(accParam)
{

    accParam = Number(accParam);

    var form = document.forms['designSpectrum'];
    var accParametersSelect = form.Acceleration.options;
    
    for(var i=0;i<accParametersSelect.length;i++)
    {
        var currentOption = accParametersSelect[i].value;
        currentOption = Number(currentOption);
        
        if(currentOption == accParam)
        {
            form.Acceleration.selectedIndex = i;
            break;
        }
    }

    updateSpectrumDiagram();
    hideAccelerationMap();
}

//klicana ob izbiri pospeska v selectu
//posodobi grafa
function accelerationSelected()
{
    updateSpectrumDiagram();
}

//updates graph if its already been drawn before
function updateSpectrumDiagram()
{
    if($("#spectrumGraphics").html() != "")
    {
        drawSpectrum();
    }
}

function prepareSapData()
{
    var sap_data = "";
    for(var i=0; i<designAnalysisData.length; i++)
    {
        var sapPoint = designAnalysisData[i];
        sap_data += formatNumber(sapPoint[0], 2, true)+"\t"+formatNumber(sapPoint[1], 8, true)+"\n";
    }
    return sap_data;
}

function prepareEavekData()
{
    var eavek_data = "";
    for(var i=0; i<designAnalysisData.length; i++)
    {
        var eavekPoint = designAnalysisData[i];
        eavek_data += formatNumber(eavekPoint[0], 2, true)+" "+formatNumber(eavekPoint[1], 8, true)+"\t";
    }
    return eavek_data;
}

String.format = function( text )
{

    //check if there are two arguments in the arguments list
    if ( arguments.length <= 1 )
    {
        //if there are not 2 or more arguments there’s nothing to replace
        //just return the original text

        return text;
    }

    //decrement to move to the second argument in the array
    var tokenCount = arguments.length - 2;

    for( var token = 0; token <= tokenCount; token++ )
    {

        //iterate through the tokens and replace their placeholders from the original text in order

        text = text.replace( new RegExp( "\\{" + token + "\\}", "gi" ), arguments[ token + 1 ] );
    }

    return text;
};
